package com.xy.controller;

import cn.hutool.core.collection.CollUtil;
import com.xy.annotate.RestMappingController;
import com.xy.dto.DeviceDataDto;
import com.xy.dto.be.MercDto;
import com.xy.service.DeviceDataServiceImpl;
import com.xy.service.be.MercService;
import com.xy.utils.MercAuthUtils;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author hechunping
 * @since 2023/3/27
 */
@RestMappingController("merc-mini/deviceData")
@AllArgsConstructor
@Api(tags = "小程序-设备销售数据统计")
public class MercMiniDeviceDataController {

    private final DeviceDataServiceImpl deviceDataService;
    private final MercService mercService;

    @PostMapping("sumPage")
    @ApiOperation("设备销售数据统计分页")
    public R<PageBean<DeviceDataDto.SumPageVo>> sumPage(@RequestBody @Valid DeviceDataDto.SumPageDto dto) {
        dto.setMercId(MercAuthUtils.getMercId());
        if (CollUtil.isEmpty(getMercDeviceIds())) {
            return R.ok(new PageBean<>());
        }
        dto.setMercDeviceIds(getMercDeviceIds());
        return R.ok(deviceDataService.sumPage(dto).getData());
    }

    @PostMapping("sumCount")
    @ApiOperation("设备销售数据统计总计")
    public R<DeviceDataDto.SumCountVo> sumCount(@RequestBody @Valid DeviceDataDto.SumCountDto dto) {
        dto.setMercId(MercAuthUtils.getMercId());
        if (CollUtil.isEmpty(getMercDeviceIds())) {
            return R.ok(new DeviceDataDto.SumCountVo());
        }
        dto.setMercDeviceIds(getMercDeviceIds());
        return R.ok(deviceDataService.sumCount(dto).getData());
    }

    /**
     * 获取商户设备列表
     *
     * @return {@link List}<{@link Long}>
     */
    public List<Long> getMercDeviceIds() {
        MercDto.MercDevicesDTO mercDevicesDTO = new MercDto.MercDevicesDTO();
        mercDevicesDTO.setMercAdmin(MercAuthUtils.isMercAdmin());
        mercDevicesDTO.setMercId(MercAuthUtils.getMercId());
        Long userId = MercAuthUtils.getMercAuth().getUserInfo().getUserId();
        mercDevicesDTO.setUserId(userId);
        List<Long> deviceIds = R.feignCheckData(mercService.getMercDevices(mercDevicesDTO));
        return deviceIds;
    }
}
