package com.xy.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.entity.DeviceData;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceDataMapper;
import com.xy.sys.EnumDataClearSize;
import com.xy.utils.DataTime;
import com.xy.utils.JobUtils;
import com.xy.utils.SysDictUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * <p>
 * 设备统计数据job
 * </p>
 *
 * @author lijin
 * @since 2023-01-11
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceDataJob {

    private DeviceDataMapper deviceDataMapper;

    /**
     * 数据清除
     *
     * @return
     */
    @XxlJob("delDeviceData")
    public ReturnT<String> delDeviceData() {
        //数据保留的天数
        SysDictRedis sysDictRedis = SysDictUtils.get(EnumDataClearSize.Code.CODE.getCode(), EnumDataClearSize.DEVICE_DATA_DAY.getCode());
        Integer jobParam = JobUtils.getPrams().gtInt();
        Integer saveDay = jobParam != null ? jobParam
                : sysDictRedis != null ? Integer.valueOf(sysDictRedis.getValue())
                : 365;
        LambdaQueryWrapper<DeviceData> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceData>()
                .le(DeviceData::getCreateTime, DataTime.getStringAround(0, 0, (~(saveDay - 1)), 0, 0, 0));
        int delete = deviceDataMapper.delete(lambdaQueryWrapper);
        if (delete > 0) {
            log.info("设备统计数据清除完成，共{}条", delete);
        }
        return ReturnT.SUCCESS;
    }
}
