package com.xy.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.DeviceConfigDto;
import com.xy.entity.DeviceConfig;
import com.xy.mapper.DeviceConfigMapper;
import com.xy.utils.Beans;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;


/**
 * <p>
 * 设备配置表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-06-16
 */
@Service
@AllArgsConstructor
@Api(tags = "设备温度告警设置")
public class DeviceConfigServiceImpl extends ServiceImpl<DeviceConfigMapper, DeviceConfig> implements DeviceConfigService {

    @PostMapping("update")
    @ApiOperation("设备温度告警阈值设置")
    public R update(@RequestBody @Validated DeviceConfigDto.Update update) {
        DeviceConfig deviceConfig = Beans.copy(DeviceConfig.class, update)
                .setUpdateTime(LocalDateTime.now());
        updateById(deviceConfig);
        return R.ok();
    }

    @PostMapping("batchUpdate")
    @ApiOperation("设备温度告警阈值批量设置")
    public R batchUpdate(@RequestBody @Validated DeviceConfigDto.BatchUpdate batchUpdate) {
        List<Long> deviceIds = batchUpdate.getDeviceIds();
        List<DeviceConfig> deviceConfigs = this.listByIds(deviceIds);
        if (CollUtil.isNotEmpty(deviceConfigs)) {
            deviceConfigs.forEach(d -> {
                d.setTempMin(batchUpdate.getTempMin());
                d.setTempMax(batchUpdate.getTempMax());
            });
            this.saveOrUpdateBatch(deviceConfigs);
        }
        return R.ok();
    }
}
