package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.dto.DeviceVersionUpDto;
import com.xy.entity.DeviceVersionUp;
import com.xy.mapper.DeviceVersionUpMapper;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;

import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 设备版本升级记录表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-06-09
 */
@Service
@AllArgsConstructor
@Api(tags = "设备版本升级记录表")
public class DeviceVersionUpServiceImpl extends ServiceImpl<DeviceVersionUpMapper, DeviceVersionUp> implements DeviceVersionUpService {

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<DeviceVersionUpDto.Vo>> page(@RequestBody DeviceVersionUpDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceVersionUp> lambdaQueryWrapper = new MybatisPlusQuery()
                .eqWrapper(page, DeviceVersionUp.class)
                .build()
                .orderByDesc(DeviceVersionUp::getCreateTime);
        IPage<DeviceVersionUp> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(DeviceVersionUpDto.Vo.class, iPage));
    }

    @ApiOperation("添加/修改")
    @PostMapping("saveOrUpdate")
    public R saveOrUpdate(@RequestBody @Validated DeviceVersionUpDto.SaveOrUpdate saveOrUpdate) {
        LambdaQueryWrapper<DeviceVersionUp> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceVersionUp>()
                .eq(DeviceVersionUp::getDeviceId, saveOrUpdate.getDeviceId())
                .eq(DeviceVersionUp::getDeviceVersion, saveOrUpdate.getDeviceVersion());
        DeviceVersionUp deviceVersionUp = getOne(lambdaQueryWrapper);
        LocalDateTime now = LocalDateTime.now();
        if (deviceVersionUp == null) {
            deviceVersionUp = Beans.copy(DeviceVersionUp.class, saveOrUpdate)
                    .setId(YitIdHelper.nextId())
                    .setCreateTime(now)
                    .setUpdateTime(now);
        } else {
            saveOrUpdate.setId(deviceVersionUp.getId());
            deviceVersionUp = Beans.copy(deviceVersionUp, saveOrUpdate).setUpdateTime(now);
            if (Emptys.check(saveOrUpdate.getAccumSendNum())) {
                deviceVersionUp.setSendNum(deviceVersionUp.getSendNum() + saveOrUpdate.getAccumSendNum());
            }
        }
        saveOrUpdate(deviceVersionUp);
        return R.ok();
    }
}