package com.xy.dto;

import cn.hutool.core.date.DateTime;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.util.List;


/**
 * 商家端小程序销售统计-更多
 */
public class MercMiniSalseCountDto {

    @Data
    @Accessors(chain = true)
    public static class UpdateInfo {
        @NotNull(message = "deviceId不能为空")
        @ApiModelProperty("设备id")
        private Long deviceId;

        @ApiModelProperty("设备名称")
        private String deviceName;

        @ApiModelProperty("点位id")
        private Long placeId;

        @ApiModelProperty("线路id")
        private Long placeLineId;

        @ApiModelProperty("区域id")
        private Long districtId;

        @ApiModelProperty("经度")
        private String lon;

        @ApiModelProperty("纬度")
        private String lat;

    }

    @Data
    @Accessors(chain = true)
    public static class Active {

        @NotNull(message = "deviceId不能为空")
        @ApiModelProperty("设备id")
        private Long deviceId;
    }

    @Data
    @Accessors(chain = true)
    public static class BusySate {

        @NotNull(message = "设备运营状态不能为空")
        @ApiModelProperty("设备运营状态 1 运营中 2 已停运")
        private Integer busyState;
        @NotNull(message = "deviceId不能为空")
        @ApiModelProperty("设备id")
        private Long deviceId;
    }


    @Data
    @Accessors(chain = true)
    public static class HomePageMonthDTO {
        @ApiModelProperty(value = "日期", required = false)
        @JsonFormat(pattern = "yyyy-MM")
        private DateTime date;

        @ApiModelProperty(value = "商户ID")
        private Long mercId;

        @ApiModelProperty(value = "拥有的设备ID", hidden = true)
        private List<Long> myDeviceIds;

        @ApiModelProperty(value = "是否商户管理员", hidden = true)
        private Boolean mercAdmin;
    }

    @Data
    @Accessors(chain = true)
    public static class DayDTO {

        @ApiModelProperty(value = "日期", required = false)
        @JsonFormat(pattern = "yyyy-MM-dd")
        private DateTime date;

        @ApiModelProperty(value = "商户ID", required = false)
        private Long mercId;

        @ApiModelProperty(value = "拥有的设备ID", hidden = true)
        private List<Long> myDeviceIds;

    }

    @Data
    @Accessors(chain = true)
    public static class MonthDTO {

        @ApiModelProperty(value = "日期", required = false)
        @JsonFormat(pattern = "yyyy-MM-dd")
        private DateTime date;

        @ApiModelProperty(value = "商户ID", required = false)
        private Long mercId;

        @ApiModelProperty(value = "拥有的设备ID", hidden = true)
        private List<Long> myDeviceIds;

    }

    @Data
    @Accessors(chain = true)
    public static class DayDetailDTO {

        @NotNull()
        @ApiModelProperty(value = "日期", required = true)
        @JsonFormat(pattern = "yyyy-MM-dd")
        private DateTime date;

    }

    @Data
    @Accessors(chain = true)
    public static class HomePageMonthVO {
        @ApiModelProperty(value = "本月收款")
        private Integer monthReceivedMoney;
        @ApiModelProperty(value = "本月销售-订单金额")
        private Integer monthOrderTotalMoney;

        @ApiModelProperty(value = "本月销售-到账金额")
        private Integer monthOrderRealMoney;
        @ApiModelProperty(value = "本月销售-数量")
        private Integer monthOrderNum;

        @ApiModelProperty(value = "本月订单-未收款金额")
        private Integer noPaymentMoney;

        @ApiModelProperty(value = "历史订单-未收款金额")
        private Integer hisNoPaymentMoney;
    }

    @Data
    @Accessors(chain = true)
    public static class SalesCountDay {
        @ApiModelProperty(value = "今日总收益=今日订单销售到账金额+今日补收合计")
        private Integer dayTotalRevenue;

        @ApiModelProperty(value = "今日订单销售-到账金额")
        private Integer dayOrderRealMoney;

        @ApiModelProperty(value = "今日订单销售-订单金额")
        private Integer dayOrderTotalMoney;

        @ApiModelProperty(value = "今日订单销售-订单数量")
        private Integer dayOrderNum;

        @ApiModelProperty(value = "今日订单销售-退款金额")
        private Integer dayRefundMoney;

        @ApiModelProperty(value = "今日订单销售-退款数量")
        private Integer dayRefundNum;

        @ApiModelProperty(value = "今日订单销售-挂账金额")
        private Integer dayPayFailedMoney;


        @ApiModelProperty(value = "今日订单销售-挂账数量")
        private Integer dayPayFailedNum;

        @ApiModelProperty(value = "今日订单销售-待确认订单数")
        private Integer dayToBeConfirmedNum;

        @ApiModelProperty(value = "今日订单销售-异常订单数量")
        private Integer dayAbnormalOrderNum;


        @ApiModelProperty(value = "历史订单今日补收-金额")
        private Integer dayHisFillMoney;
        @ApiModelProperty(value = "历史订单今日补收-订单数量")
        private Integer dayHisFillNum;


        @ApiModelProperty(value = "历史订单今日补退-订单金额")
        private Integer dayHisRefundMoney;

        @ApiModelProperty(value = "历史订单今日补退-订单数量")
        private Integer dayHisRefundNum;


        @ApiModelProperty(value = "风险单-待处理数量")
        private Integer toDoRiskCount;

        @ApiModelProperty(value = "风险单-今日处理数量")
        private Integer dayRiskHandleCount;
        @ApiModelProperty(value = "风险单-今日新增数量")
        private Integer dayRiskAddCount;


    }


    @Data
    @Accessors(chain = true)
    public static class MonthCountVO {
        @ApiModelProperty(value = "本月总收益")
        private Integer monthTotalRevenue;

        @ApiModelProperty(value = "本月收益列表")
        private List<DayRevenue> revenueList;


    }

    @Data
    @Accessors(chain = true)
    public static class DayRevenue {
        @ApiModelProperty(value = "日收益")
        private Integer dayTotalRevenue;
        @ApiModelProperty(value = "日期")
        private String date;

    }


}
